%global commit0 01a0c815d1a98eb9b38341cf63546f234fbcc43b
%global shortcommit0 %(c=%{commit0}; echo ${c:0:7})
%global date0 20230824
# MPI build based on spec file dbcsr by Orion
%global mpi_list openmpi mpich
# $mpi will be evaluated in the loops below
%global _vpath_builddir %{_vendor}-%{_target_os}-build-${mpi:-serial}

Summary:        Communications library for AI/ML
Name:           gloo
License:        BSD-3-Clause
Version:        0.5.0^git%{date0}.%{shortcommit0}
Release:        1%{?dist}

URL:            https://github.com/facebookincubator/%{name}
Source0:        %{url}/archive/%{commit0}/%{name}-%{shortcommit0}.tar.gz
Patch0:         0001-gloo-fedora-cmake-changes.patch
Patch1:         0002-gloo-fedora-cmake-libuv.patch
# See CMakeLists, gloo only builds on 64 bit systems
ExcludeArch:    i686

BuildRequires:  cmake
BuildRequires:  gcc-c++
BuildRequires:  hiredis-devel
BuildRequires:  libibverbs
BuildRequires:  libuv-devel
BuildRequires:  libuv-static
BuildRequires:  rdma-core-devel
BuildRequires:  redis-devel
# Needed for tests
BuildRequires:  gmock-devel
BuildRequires:  gtest-devel
BuildRequires:  openssl1.1-devel
Requires:       %{name}-docs

%global base_description \
Gloo is a collective communications library. It comes with a number of\
collective algorithms useful for machine learning applications. These\
include a barrier, broadcast, and allreduce.\
\
Transport of data between participating machines is abstracted so that\
IP can be used at all times, or InifiniBand (or RoCE) when available.\
In the latter case, if the InfiniBand transport is used, GPUDirect can\
be used to accelerate cross machine GPU-to-GPU memory transfers.\
\
Where applicable, algorithms have an implementation that works with\
system memory buffers, and one that works with NVIDIA GPU memory buffers.\
In the latter case, it is not necessary to copy memory between host and\
device; this is taken care of by the algorithm implementations.

%description
%{base_description}

%package docs
Summary: Documentation and license for %{name}
Buildarch: noarch

%description docs
Documentation only package used by other packages for
%{name}.

%package devel

Summary:        Headers and libraries for %{name}
Requires:       %{name}%{?_isa} = %{version}-%{release}

%description devel
Libraries and headers for developing applications that
use %{name}.

%package openmpi
Summary: Gloo - openmpi version
BuildRequires:  openmpi-devel
Requires: %{name}-docs

%description openmpi
%{base_description}

This package can use openmpi as a backend.

%package openmpi-devel
Summary:   Development libraries and headers
Requires: %{name}-openmpi%{?_isa} = %{version}-%{release}

%description openmpi-devel
Libraries and header files for developing applications
that use %{name}-openmpi.

%package mpich
Summary: Gloo - mpich version
BuildRequires:  mpich-devel
Requires: %{name}-docs

%description mpich
%{base_description}

This package can use mpich as a backend.

%package mpich-devel
Summary: Development libraries and headers
BuildRequires:  mpich-devel
Requires: %{name}-mpich%{?_isa} = %{version}-%{release}

%description mpich-devel
Libraries and header files for developing applications
that use %{name}-mpich.

%prep
%autosetup -p1 -n %{name}-%{commit0}

%build
%cmake -DCMAKE_CXX_STANDARD=14 \
       -DBUILD_TEST=ON \
       -DUSE_IBVERBS=ON \
       -DUSE_LIBUV=ON \
       -DUSE_TCP_OPENSSL_LINK=ON \
       -DUSE_REDIS=ON \
       -DUSE_MPI=OFF \
       -DBUILD_EXAMPLES=ON \
       -DCMAKE_INSTALL_PREFIX=%{_prefix} \
       -DCMAKE_INSTALL_LIBDIR=%{_libdir}
%cmake_build
for mpi in %{mpi_list}
do
  module load mpi/$mpi-%{_arch}
  %cmake -DCMAKE_CXX_STANDARD=14 \
       -DBUILD_TEST=ON \
       -DUSE_IBVERBS=ON \
       -DUSE_LIBUV=ON \
       -DUSE_TCP_OPENSSL_LINK=ON \
       -DUSE_REDIS=ON \
       -DUSE_MPI=ON \
       -DBUILD_EXAMPLES=ON \
       -DCMAKE_INSTALL_PREFIX=$MPI_HOME \
       -DCMAKE_INSTALL_LIBDIR=lib
   %cmake_build
   module purge
done

%install
%cmake_install
mkdir -p %{buildroot}%{_datadir}/Gloo/docs
cp -p -r docs/* %{buildroot}%{_datadir}/Gloo/docs
for mpi in %{mpi_list}
do
  module load mpi/$mpi-%{_arch}
  %cmake_install
  module purge
done

%check
# Test needs to generate an SSL certificate, which fails
# /%{__cmake_builddir}/gloo/test/gloo_test 
# Smoke test, need to figure out how to get looks_like_mpi to run
PREFIX=test1 SIZE=1 RANK=0 ./%{__cmake_builddir}/gloo/examples/example1

%files docs
%license LICENSE
%doc README.md
%dir %{_datadir}/Gloo
%doc %{_datadir}/Gloo/docs/

%files
%{_libdir}/lib%{name}.so.23.*

%files devel
%dir %{_includedir}/%{name}
%{_includedir}/%{name}/*.h
%dir %{_includedir}/%{name}/common
%{_includedir}/%{name}/common/*.h
%dir %{_includedir}/%{name}/rendezvous
%{_includedir}/%{name}/rendezvous/*.h
%dir %{_includedir}/%{name}/transport
%{_includedir}/%{name}/transport/*.h
%dir %_includedir/%{name}/transport/tcp
%{_includedir}/%{name}/transport/tcp/*.h
%dir %{_includedir}/%{name}/transport/tcp/tls
%{_includedir}/%{name}/transport/tcp/tls/*.h
%dir %{_includedir}/%{name}/transport/ibverbs
%{_includedir}/%{name}/transport/ibverbs/*.h
%dir %{_includedir}/%{name}/transport/uv
%{_includedir}/%{name}/transport/uv/*.h
%dir %{_libdir}/cmake/Gloo
%{_libdir}/cmake/Gloo/*.cmake
%{_libdir}/lib%{name}.so

%files openmpi
%{_libdir}/openmpi/lib/lib%{name}.so.23.*

%files openmpi-devel
%dir %{_libdir}/openmpi/include/%{name}
%{_libdir}/openmpi/include/%{name}/*.h
%dir %{_libdir}/openmpi/include/%{name}/common
%{_libdir}/openmpi/include/%{name}/common/*.h
%dir %{_libdir}/openmpi/include/%{name}/rendezvous
%{_libdir}/openmpi/include/%{name}/rendezvous/*.h
%dir %{_libdir}/openmpi/include/%{name}/transport
%{_libdir}/openmpi/include/%{name}/transport/*.h
%dir %{_libdir}/openmpi/include/%{name}/transport/tcp
%{_libdir}/openmpi/include/%{name}/transport/tcp/*.h
%dir %{_libdir}/openmpi/include/%{name}/transport/tcp/tls
%{_libdir}/openmpi/include/%{name}/transport/tcp/tls/*.h
%dir %{_libdir}/openmpi/include/%{name}/transport/ibverbs
%{_libdir}/openmpi/include/%{name}/transport/ibverbs/*.h
%dir %{_libdir}/openmpi/include/%{name}/transport/uv
%{_libdir}/openmpi/include/%{name}/transport/uv/*.h
%dir %{_libdir}/openmpi/include/%{name}/mpi
%{_libdir}/openmpi/include/%{name}/mpi/*.h
%dir %{_libdir}/openmpi/lib/cmake/Gloo
%{_libdir}/openmpi/lib/cmake/Gloo/*.cmake
%{_libdir}/openmpi/lib/lib%{name}.so

%files mpich
%{_libdir}/mpich/lib/lib%{name}.so.23.*

%files mpich-devel
%dir %{_libdir}/mpich/include/%{name}
%{_libdir}/mpich/include/%{name}/*.h
%dir %{_libdir}/mpich/include/%{name}/common
%{_libdir}/mpich/include/%{name}/common/*.h
%dir %{_libdir}/mpich/include/%{name}/rendezvous
%{_libdir}/mpich/include/%{name}/rendezvous/*.h
%dir %{_libdir}/mpich/include/%{name}/transport
%{_libdir}/mpich/include/%{name}/transport/*.h
%dir %{_libdir}/mpich/include/%{name}/transport/tcp
%{_libdir}/mpich/include/%{name}/transport/tcp/*.h
%dir %{_libdir}/mpich/include/%{name}/transport/tcp/tls
%{_libdir}/mpich/include/%{name}/transport/tcp/tls/*.h
%dir %{_libdir}/mpich/include/%{name}/transport/ibverbs
%{_libdir}/mpich/include/%{name}/transport/ibverbs/*.h
%dir %{_libdir}/mpich/include/%{name}/transport/uv
%{_libdir}/mpich/include/%{name}/transport/uv/*.h
%dir %{_libdir}/mpich/include/%{name}/mpi
%{_libdir}/mpich/include/%{name}/mpi/*.h
%dir %{_libdir}/mpich/lib/cmake/Gloo
%{_libdir}/mpich/lib/cmake/Gloo/*.cmake
%{_libdir}/mpich/lib/lib%{name}.so


%changelog
* Fri Sep 22 2023 Tom Rix <trix@redhat.com> - 0.5.0^git20230824.01a0c81-1
- Initial package
